require 'json'
require 'uri'
require 'net/http'
# see the URL below for information on how to write OpenStudio measures
# http://nrel.github.io/OpenStudio-user-documentation/measures/measure_writing_guide/
# $servidor = 'http://127.0.0.1:8000'
$servidor = 'http://www.certificomivivienda.es'
# $servidor = nil
# start the measure
class FijarCoeficientesDePaso < OpenStudio::Ruleset::WorkspaceUserScript

  # human readable name
  def name
    return "Fijar coeficientes de paso"
  end

  # human readable description
  def description
    return "Cambio de los coeficientes de paso a energia primaria por los que figuran en la HU."
  end

  # human readable description of modeling approach
  def modeler_description
    return ""
  end

  # define the arguments that the user will input
  def arguments(workspace)
    args = OpenStudio::Ruleset::OSArgumentVector.new

    # the name of the zone to add to the model
    # zone_name = OpenStudio::Ruleset::OSArgument.makeStringArgument("zone_name", true)
    # zone_name.setDisplayName("New zone name")
    # zone_name.setDescription("This name will be used as the name of the new zone.")
    # args << zone_name

    return args
  end 

  # define what happens when the measure is run
  def run(workspace, runner, user_arguments)
    super(workspace, runner, user_arguments)

    # use the built-in error checking 
    if !runner.validateUserArguments(arguments(workspace), user_arguments)
      return false
    end

    # reporting initial condition of model
    runner.registerInitialCondition("Comenzamos...")
    
    model = runner.lastOpenStudioModel
    if model.empty?
      runner.registerError("Cannot find last model.")
      return false
    end
    model = model.get


    extraPeninsular = JSON.parse(model.building.get.comment[2..-1])['extraPeninsular']
    provincia = '' # Es necesario para los nuevos coeficientes de paso

    # Eliminamos los que existen
    listaInstancia = workspace.getObjectsByType("FuelFactors".to_IddObjectType)
    listaInstancia.each {|i| i.remove()}

    dicCoeficientesDePaso = coeficientesDePaso(extraPeninsular,provincia)

    formatoTabla_string = "    
FuelFactors,
    Electricity,             !- Existing Fuel Resource Name
    kWh,                     !- Units of Measure
    ,                        !- Energy per Unit Factor
    #{dicCoeficientesDePaso['Electricity']['energiaPrimariaNoRenovable']},                   !- Source Energy Factor {J/J}
    ,                        !- Source Energy Schedule Name
    #{dicCoeficientesDePaso['Electricity']['emisionesCO2']*1000.0 / 3.6};                   !- CO2 Emission Factor {g/MJ}
      "      
    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
    object = idfObject.get
    wsObject = workspace.addObject(object)


    formatoTabla_string = "    
FuelFactors,
    Natural Gas,             !- Existing Fuel Resource Name
    kWh,                     !- Units of Measure
    ,                        !- Energy per Unit Factor
    #{dicCoeficientesDePaso['Natural Gas']['energiaPrimariaNoRenovable']},                   !- Source Energy Factor {J/J}
    ,                        !- Source Energy Schedule Name
    #{dicCoeficientesDePaso['Natural Gas']['emisionesCO2']*1000.0 / 3.6};                   !- CO2 Emission Factor {g/MJ}
      "      
    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
    object = idfObject.get
    wsObject = workspace.addObject(object)



    formatoTabla_string = "    
FuelFactors,
    Diesel,             !- Existing Fuel Resource Name
    kWh,                     !- Units of Measure
    ,                        !- Energy per Unit Factor
    #{dicCoeficientesDePaso['Diesel']['energiaPrimariaNoRenovable']},                   !- Source Energy Factor {J/J}
    ,                        !- Source Energy Schedule Name
    #{dicCoeficientesDePaso['Diesel']['emisionesCO2']*1000.0 / 3.6};                   !- CO2 Emission Factor {g/MJ}
      "      
    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
    object = idfObject.get
    wsObject = workspace.addObject(object)


#    formatoTabla_string = "    
#FuelFactors,
#    FuelOil#1,             !- Existing Fuel Resource Name
#    kWh,                     !- Units of Measure
#    ,                        !- Energy per Unit Factor
#    #{dicCoeficientesDePaso['Fuel Oil #1']['energiaPrimariaNoRenovable']},                   !- Source Energy Factor {J/J}
#    ,                        !- Source Energy Schedule Name
#    #{dicCoeficientesDePaso['Fuel Oil #1']['emisionesCO2']*1000.0 / 3.6};                   !- CO2 Emission Factor {g/MJ}
#      "      
#    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
#    object = idfObject.get
#    wsObject = workspace.addObject(object)

#    formatoTabla_string = "    
#FuelFactors,
#    FuelOil#2,             !- Existing Fuel Resource Name
#    kWh,                     !- Units of Measure
#    ,                        !- Energy per Unit Factor
#    #{dicCoeficientesDePaso['Fuel Oil #2']['energiaPrimariaNoRenovable']},                  !- Source Energy Factor {J/J}
#    ,                        !- Source Energy Schedule Name
#    #{dicCoeficientesDePaso['Fuel Oil #2']['emisionesCO2']*1000.0 / 3.6};                  !- CO2 Emission Factor {g/MJ}
#      "      
#    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
#    object = idfObject.get
#    wsObject = workspace.addObject(object)
    
    formatoTabla_string = "        
FuelFactors,
    Propane,             !- Existing Fuel Resource Name
    kWh,                     !- Units of Measure
    ,                        !- Energy per Unit Factor
    #{dicCoeficientesDePaso['Propane']['energiaPrimariaNoRenovable']},                   !- Source Energy Factor {J/J}
    ,                        !- Source Energy Schedule Name
    #{dicCoeficientesDePaso['Propane']['emisionesCO2']*1000.0 / 3.6};                   !- CO2 Emission Factor {g/MJ}
      "      
    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
    object = idfObject.get
    wsObject = workspace.addObject(object)    
    
    formatoTabla_string = "        
FuelFactors,
    Coal,             !- Existing Fuel Resource Name
    kWh,                     !- Units of Measure
    ,                        !- Energy per Unit Factor
    #{dicCoeficientesDePaso['Coal']['energiaPrimariaNoRenovable']},                   !- Source Energy Factor {J/J}
    ,                        !- Source Energy Schedule Name
    #{dicCoeficientesDePaso['Coal']['emisionesCO2']*1000.0 / 3.6};                   !- CO2 Emission Factor {g/MJ}
      "      
    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
    object = idfObject.get
    wsObject = workspace.addObject(object)        



    formatoTabla_string = "        
FuelFactors,
    OtherFuel1,             !- Existing Fuel Resource Name
    kWh,                     !- Units of Measure
    ,                        !- Energy per Unit Factor
    #{dicCoeficientesDePaso['Other Fuel 1']['energiaPrimariaNoRenovable']},                   !- Source Energy Factor {J/J}
    ,                        !- Source Energy Schedule Name
    #{dicCoeficientesDePaso['Other Fuel 1']['emisionesCO2']*1000.0 / 3.6};                   !- CO2 Emission Factor {g/MJ}
      "      
    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
    object = idfObject.get
    wsObject = workspace.addObject(object)     

    formatoTabla_string = "        
FuelFactors,
    OtherFuel2,             !- Existing Fuel Resource Name
    kWh,                     !- Units of Measure
    ,                        !- Energy per Unit Factor
    #{dicCoeficientesDePaso['Other Fuel 2']['energiaPrimariaNoRenovable']},                   !- Source Energy Factor {J/J}
    ,                        !- Source Energy Schedule Name
    #{dicCoeficientesDePaso['Other Fuel 2']['emisionesCO2']*1000.0 / 3.6};                   !- CO2 Emission Factor {g/MJ}
      "      
    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
    object = idfObject.get
    wsObject = workspace.addObject(object)   




    formatoTabla_string = "    
EnvironmentalImpactFactors,
    0.92,                     !- District Heating Efficiency
    2.0,                       !- District Cooling COP {W/W}
    1.0,                    !- Steam Conversion Efficiency
    0.,                 !- Total Carbon Equivalent Emission Factor From N2O {kg/kg}
    0.,                  !- Total Carbon Equivalent Emission Factor From CH4 {kg/kg}
    #{dicCoeficientesDePaso['District Heating']['emisionesCO2']*1000.0 / 3.6};                  !- Total Carbon Equivalent Emission Factor From CO2 {kg/kg}"      
    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
    object = idfObject.get
    wsObject = workspace.addObject(object)






    # report final condition of model
    runner.registerFinalCondition("Fin")
    
    return true
 
  end
  
def coeficientesDePaso(extraPeninsular,provincia)
  begin
    uri = URI.parse("#{$servidor}/energyPlus/coeficientesDePaso/")
    response = Net::HTTP.post_form(uri, {"extraPeninsular" => extraPeninsular,
                 "provincia" => provincia})
  
    # Llega un diccionario con la energia semanal y el numero de horas
    dicCoeficientesDePaso = JSON.parse(response.body)    
  rescue
    dicCoeficientesDePaso = JSON.parse('{"District Heating": {"energiaPrimariaTotal": 1.099, "traduccion": "Calefacci\u00f3n de distrito", "energiaPrimariaNoRenovable": 1.099, "emisionesCO2": 0.222}, "Fuel Oil #2": {"energiaPrimariaTotal": 1.182, "traduccion": "Fuel Oil #2", "energiaPrimariaNoRenovable": 1.179, "emisionesCO2": 0.311}, "Natural Gas": {"energiaPrimariaTotal": 1.195, "traduccion": "Gas Natural", "energiaPrimariaNoRenovable": 1.19, "emisionesCO2": 0.252}, "Electricity": {"energiaPrimariaTotal": 2.368, "traduccion": "Electricidad", "energiaPrimariaNoRenovable": 1.954, "emisionesCO2": 0.331}, "Diesel": {"energiaPrimariaTotal": 1.182, "traduccion": "Gasoleo", "energiaPrimariaNoRenovable": 1.179, "emisionesCO2": 0.311}, "Fuel Oil #1": {"energiaPrimariaTotal": 1.182, "traduccion": "Fuel Oil #1", "energiaPrimariaNoRenovable": 1.179, "emisionesCO2": 0.311}, "Gasoline": {"energiaPrimariaTotal": 1.081, "traduccion": "Gasolina", "energiaPrimariaNoRenovable": 1.081, "emisionesCO2": 0.287}, "District Cooling": {"energiaPrimariaTotal": 1.301, "traduccion": "Refrigeraci\u00f3n de distrito", "energiaPrimariaNoRenovable": 1.301, "emisionesCO2": 0.324}, "Coal": {"energiaPrimariaTotal": 1.084, "traduccion": "Carb\u00f3n", "energiaPrimariaNoRenovable": 1.082, "emisionesCO2": 0.472}, "Other Fuel 2": {"energiaPrimariaTotal": 1.037, "traduccion": "Biomasa (otro tipo)", "energiaPrimariaNoRenovable": 0.034, "emisionesCO2": 0.018}, "Other Fuel 1": {"energiaPrimariaTotal": 1.113, "traduccion": "Biomasa densificada (pelets)", "energiaPrimariaNoRenovable": 0.085, "emisionesCO2": 0.018}, "Steam": {"energiaPrimariaTotal": 0.3, "traduccion": "Vapor", "energiaPrimariaNoRenovable": 0.3, "emisionesCO2": 0.0}, "Propane": {"energiaPrimariaTotal": 1.204, "traduccion": "GLP", "energiaPrimariaNoRenovable": 1.201, "emisionesCO2": 0.254}}') 
  end


  return dicCoeficientesDePaso
end 

end 

# register the measure to be used by the application
FijarCoeficientesDePaso.new.registerWithApplication
