# see the URL below for information on how to write OpenStudio measures
# http://nrel.github.io/OpenStudio-user-documentation/measures/measure_writing_guide/
# $servidor = 'http://127.0.0.1:8000'
$servidor = 'http://www.certificomivivienda.es'
# start the measure
class NewMeasure < OpenStudio::Ruleset::WorkspaceUserScript

  # human readable name
  def name
    return "New Measure"
  end

  # human readable description
  def description
    return "Vamos a intentar que el informe salga en kWh."
  end

  # human readable description of modeling approach
  def modeler_description
    return "Replace this text with an explanation for the energy modeler specifically.  It should explain how the measure is modeled, including any requirements about how the baseline model must be set up, major assumptions, citations of references to applicable modeling resources, etc.  The energy modeler should be able to read this description and understand what changes the measure is making to the model and why these changes are being made.  Because the Modeler Description is written for an expert audience, using common abbreviations for brevity is good practice."
  end

  # define the arguments that the user will input
  def arguments(workspace)
    args = OpenStudio::Ruleset::OSArgumentVector.new

    # the name of the zone to add to the model
    # zone_name = OpenStudio::Ruleset::OSArgument.makeStringArgument("zone_name", true)
    # zone_name.setDisplayName("New zone name")
    # zone_name.setDescription("This name will be used as the name of the new zone.")
    # args << zone_name

    return args
  end 

  # define what happens when the measure is run
  def run(workspace, runner, user_arguments)
    super(workspace, runner, user_arguments)

    # use the built-in error checking 
    if !runner.validateUserArguments(arguments(workspace), user_arguments)
      return false
    end





    # reporting initial condition of model
    runner.registerInitialCondition("Comenzamos...")


    # Eliminamos los que existen
    listaInstancia = workspace.getObjectsByType("OutputControl_Table_Style".to_IddObjectType)

    listaInstancia.each {|i| i.remove()}


    # add a new zone to the model with the new name
    # http://apps1.eere.energy.gov/buildings/energyplus/pdfs/inputoutputreference.pdf#nameddest=Zone

    formatoTabla_string = "    
    OutputControl:Table:Style,
      HTML,
      JtoKWH;
      "      
    idfObject = OpenStudio::IdfObject::load(formatoTabla_string)
    object = idfObject.get
    wsObject = workspace.addObject(object)
   #  new_zone = wsObject.get

    # echo the new zone's name back to the user, using the index based getString method
    runner.registerInfo("Ya esta el lio armado.")

    # report final condition of model
    runner.registerFinalCondition("To guapo...")
    
    return true
 
  end

end 

# register the measure to be used by the application
NewMeasure.new.registerWithApplication
