# see the URL below for information on how to write OpenStudio measures
# http://nrel.github.io/OpenStudio-user-documentation/reference/measure_writing_guide/

# start the measure
class IntroducePropiedadesMaterialesDeCambioDeFase < OpenStudio::Ruleset::WorkspaceUserScript

  # human readable name
  def name
    return "Introduce propiedades materiales de cambio de fase"
  end

  # human readable description
  def description
    return ""
  end

  # human readable description of modeling approach
  def modeler_description
    return ""
  end

  # define the arguments that the user will input
  def arguments(workspace)
    args = OpenStudio::Ruleset::OSArgumentVector.new

    # Argumentos
    
    materialAlQueSeAplica = OpenStudio::Ruleset::OSArgument.makeStringArgument("materialAlQueSeAplica", true)
    materialAlQueSeAplica.setDisplayName("Material standart al que se aplican las propiedades de cambio de fase")
    materialAlQueSeAplica.setDescription("")
    materialAlQueSeAplica.setDefaultValue("Placa de yeso laminado: &#10Placo: PLACO PHONIQUE 15 (0.015 m)&#10R: 0.1 m2K/W")
    args << materialAlQueSeAplica    
    
    
    coeficienteTemperaturaConductividadTermica = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("coeficienteTemperaturaConductividadTermica", true)
    coeficienteTemperaturaConductividadTermica.setDisplayName("Coeficiente variación de la k en funcion de la temperatura")
    coeficienteTemperaturaConductividadTermica.setDescription("This field is used to enter the temperature dependent coefficient for thermal conductivity of the material. Units for this parameter are (W/(m-K2). This is the thermal conductivity change per unit temperature excursion from 20 C. The conductivity value at 20 C is the one specified with the basic material properties of the regular material specified in the name field. The thermal conductivity is obtained from:")
    coeficienteTemperaturaConductividadTermica.setDefaultValue(0.0)
    args << coeficienteTemperaturaConductividadTermica

    temperatura1 = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("temperatura1", true)
    temperatura1.setDisplayName("Temperatura 1 (ºC)")
    temperatura1.setDescription("")
    temperatura1.setDefaultValue(-20.0)
    args << temperatura1

    entalpia1 = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("entalpia1", true)
    entalpia1.setDisplayName("Entalpía 1 (J/kg)")
    entalpia1.setDescription("")
    entalpia1.setDefaultValue(0.01)
    args << entalpia1


    temperatura2 = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("temperatura2", true)
    temperatura2.setDisplayName("Temperatura 2 (ºC)")
    temperatura2.setDescription("")
    temperatura2.setDefaultValue(20.0)
    args << temperatura2

    entalpia2 = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("entalpia2", true)
    entalpia2.setDisplayName("Entalpía 2 (J/kg)")
    entalpia2.setDescription("")
    entalpia2.setDefaultValue(33400)
    args << entalpia2
    
    temperatura3 = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("temperatura3", true)
    temperatura3.setDisplayName("Temperatura 3 (ºC)")
    temperatura3.setDescription("")
    temperatura3.setDefaultValue(20.5)
    args << temperatura3

    entalpia3 = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("entalpia3", true)
    entalpia3.setDisplayName("Entalpía 3 (J/kg)")
    entalpia3.setDescription("")
    entalpia3.setDefaultValue(70000)
    args << entalpia3
    
    temperatura4 = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("temperatura4", true)
    temperatura4.setDisplayName("Temperatura 4 (ºC)")
    temperatura4.setDescription("")
    temperatura4.setDefaultValue(100.0)
    args << temperatura4

    entalpia4 = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("entalpia4", true)
    entalpia4.setDisplayName("Entalpía 4 (J/kg)")
    entalpia4.setDescription("")
    entalpia4.setDefaultValue(137000)
    args << entalpia4    


    return args
  end 

  # define what happens when the measure is run
  def run(workspace, runner, user_arguments)
    super(workspace, runner, user_arguments)

    # use the built-in error checking 
    if !runner.validateUserArguments(arguments(workspace), user_arguments)
      return false
    end

    # assign the user inputs to variables
    materialAlQueSeAplica = runner.getStringArgumentValue("materialAlQueSeAplica", user_arguments)
    
    coeficienteTemperaturaConductividadTermica = runner.getDoubleArgumentValue("coeficienteTemperaturaConductividadTermica", user_arguments)
    temperatura1 = runner.getDoubleArgumentValue("temperatura1", user_arguments)
    entalpia1 = runner.getDoubleArgumentValue("entalpia1", user_arguments)
    temperatura2 = runner.getDoubleArgumentValue("temperatura2", user_arguments)
    entalpia2 = runner.getDoubleArgumentValue("entalpia2", user_arguments)
    temperatura3 = runner.getDoubleArgumentValue("temperatura3", user_arguments)
    entalpia3 = runner.getDoubleArgumentValue("entalpia3", user_arguments)
    temperatura4 = runner.getDoubleArgumentValue("temperatura4", user_arguments)
    entalpia4 = runner.getDoubleArgumentValue("entalpia4", user_arguments)


    
    
    
    # cambiamos el algortimo de calculo, PCM se tienen que calcular con conductionFiniteDifferene
    algoritmo = workspace.getObjectsByType("HeatBalanceAlgorithm".to_IddObjectType)
    modoInicial = algoritmo[0].getString(0)
    algoritmo[0].setString(0,"ConductionFiniteDifference")
    


    # reporting initial condition of model
    runner.registerInitialCondition("Se cambia: HeatBalanceAlgorithm de #{modoInicial} a  ConductionFiniteDifference.")


    propiedadesCambioDeFaseString = "
  MaterialProperty:PhaseChange,
    #{materialAlQueSeAplica},  !- Name
      #{coeficienteTemperaturaConductividadTermica},             !- Temperature coefficient,thermal conductivity(W/m K2)
      #{temperatura1},            !- Temperature 1, C
      #{entalpia1},            !- Enthalpy 1 at –20C, (J/kg)
       #{temperatura2},             !- Temperature 2, C
      #{entalpia2},           !- Enthalpy 2, (J/kg)
       #{temperatura3},            !- temperature 3, C
      #{entalpia3},           !- Enthalpy 3, (J/kg)
       #{temperatura4},            !- Temperature 4, C
      #{entalpia4};          !- Enthalpy 4, (J/kg)
        "
    idfObject = OpenStudio::IdfObject::load(propiedadesCambioDeFaseString)
    object = idfObject.get
    wsObject = workspace.addObject(object)





    runner.registerFinalCondition("Introducidas las propiedades del PCM.")
    
    return true
 
  end

end 

# register the measure to be used by the application
IntroducePropiedadesMaterialesDeCambioDeFase.new.registerWithApplication
