# see the URL below for information on how to write OpenStudio measures
# http://nrel.github.io/OpenStudio-user-documentation/reference/measure_writing_guide/

# start the measure
class IntroduceUnaFachadaVentilada < OpenStudio::Ruleset::WorkspaceUserScript

  # human readable name
  def name
    return "Introduce una fachada ventilada"
  end

  # human readable description
  def description
    return ""
  end

  # human readable description of modeling approach
  def modeler_description
    return ""
  end

  # define the arguments that the user will input
  def arguments(workspace)
    args = OpenStudio::Ruleset::OSArgumentVector.new

    # Argumentos
    fraccionAreaAperturas = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("fraccionAreaAperturas", true)
    fraccionAreaAperturas.setDisplayName("Fraccion area aperturas")
    fraccionAreaAperturas.setDescription("This field is used to enter an area fraction for what part of the baffle consists of openings. The area of the openings will set to the product of this field and the sum of the area of the underlying surfaces.")
    fraccionAreaAperturas.setDefaultValue(0.02)
    args << fraccionAreaAperturas

    emisividadPlacaExterior = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("emisividadPlacaExterior", true)
    emisividadPlacaExterior.setDisplayName("Emisividad de la placa exterior")
    emisividadPlacaExterior.setDescription("This field is used to enter the thermal emissivity of the baffler. This surface property is for longwave infrared radiation. The property is used for both sides of collector. Most painted materials have an emissivity of 0.9.")
    emisividadPlacaExterior.setDefaultValue(0.9)
    args << emisividadPlacaExterior

    absortividadSolarPlacaExterior = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("absortividadSolarPlacaExterior", true)
    absortividadSolarPlacaExterior.setDisplayName("Absortividad solar de la placa exterior")
    absortividadSolarPlacaExterior.setDescription("This field is used to enter the solar absorbtivity of the baffle. This surface property is for shortwave, solar radiation. The property is used for the front side of the baffle that faces the environment. Darker colors have a higher absorptivity. While black is the highest performance, other colors might be used to match the color scheme of the rest of the façade.")
    absortividadSolarPlacaExterior.setDefaultValue(0.92)
    args << absortividadSolarPlacaExterior

    alturaMediaParaVentilacionPorFlotabilidad = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("alturaMediaParaVentilacionPorFlotabilidad", true)
    alturaMediaParaVentilacionPorFlotabilidad.setDisplayName("Altura media para la ventilación por flotabilidad [m]")
    alturaMediaParaVentilacionPorFlotabilidad.setDescription("This field is used to enter a nominal height scale (m) for prediction of ventilation induced by bouancy. This value (ΔHNPL) is defined as the height from the midpoint of the lower opening to the neutral pressure level. Increasing the value will increase the ventilation rate due to buoyancy.")
    alturaMediaParaVentilacionPorFlotabilidad.setDefaultValue(1.5)
    args << alturaMediaParaVentilacionPorFlotabilidad


    espesorCavidad = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("espesorCavidad", true)
    espesorCavidad.setDisplayName("Espesor de la cavidad [m]")
    espesorCavidad.setDescription("This field is used to enter a nominal gap thickness (m) for the collector. If the baffle is corrugated, use the average depth. This distance value is only used when the collector is near horizontal to determine a length scale in the vertical direction for buoyancy calculations. For example, if the collector is mounted on a flat roof, its tilt-adjusted height is zero and the program will use this gap thickness as a length scale rather than the height from the previous field.")
    espesorCavidad.setDefaultValue(0.05)
    args << espesorCavidad


    ratioAreaProyectada = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("ratioAreaProyectada", true)
    ratioAreaProyectada.setDisplayName("Ratio del area proyectada")
    ratioAreaProyectada.setDescription("This field is used to enter a factor that accounts for the extra surface area resulting from and uneven baffle surface. Corrugations may be present to help stiffen the baffle or ventilated roofing tiles may have more surface are for convection heat transfer than the underlying surface. The projected surface area is obtained by the program from the (flat) underlying surfaces. If the baffle is flat then this ratio is 1.0. If the baffle is corrugated, then this ratio will be greater than one with a typical value might be 1.165.")
    ratioAreaProyectada.setDefaultValue(1.0)
    args << ratioAreaProyectada
    
    efectividadHuecosRespectoAlViento = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("efectividadHuecosRespectoAlViento", true)
    efectividadHuecosRespectoAlViento.setDisplayName("Efectividad de los huecos respecto al viento")
    efectividadHuecosRespectoAlViento.setDescription("This field is used to enter a value for the coefficient used to determine natural air exchanges from wind. Wind will cause exterior air to move in and out of the cavity. Cv is an arbitrary coefficient used to model the effectiveness of openings and depends on opening geometry and the orientation with respect to the wind. Cv should probably be in the range 0.05 to 0.65. Increasing Cv will increase the amount of natural ventilation. The following equation shows how Cv is used in the program to predict the volumetric flow rate due to wind:")
    efectividadHuecosRespectoAlViento.setDefaultValue(0.1)
    args << efectividadHuecosRespectoAlViento    


    coeficientesDescargaHuecosFlotabilidad = OpenStudio::Ruleset::OSArgument.makeDoubleArgument("coeficientesDescargaHuecosFlotabilidad", true)
    coeficientesDescargaHuecosFlotabilidad.setDisplayName("Coeficiente de descarga de los huecos para la flotabilidad")
    coeficientesDescargaHuecosFlotabilidad.setDescription("This field is used to enter a value for the coefficient used to determine natural air exchanges from buoyancy. Stack or buoyancy effects will cause exterior air to move in and out of the cavity. Cd is an arbitrary discharge coefficient that depends on the geometry of the opening. Cd should probably be in the range 0.1 to 1.0. Increasing Cd will increase the amount of natural ventilation. The following equations show how Cd is used in the program to predict the volume flow rate due to buoyancy:")
    coeficientesDescargaHuecosFlotabilidad.setDefaultValue(0.5)
    args << coeficientesDescargaHuecosFlotabilidad    


    return args
  end 

  # define what happens when the measure is run
  def run(workspace, runner, user_arguments)
    super(workspace, runner, user_arguments)

    # use the built-in error checking 
    if !runner.validateUserArguments(arguments(workspace), user_arguments)
      return false
    end

    # assign the user inputs to variables
    fraccionAreaAperturas = runner.getDoubleArgumentValue("fraccionAreaAperturas", user_arguments)
    emisividadPlacaExterior = runner.getDoubleArgumentValue("emisividadPlacaExterior", user_arguments)
    absortividadSolarPlacaExterior = runner.getDoubleArgumentValue("absortividadSolarPlacaExterior", user_arguments)
    alturaMediaParaVentilacionPorFlotabilidad = runner.getDoubleArgumentValue("alturaMediaParaVentilacionPorFlotabilidad", user_arguments)
    espesorCavidad = runner.getDoubleArgumentValue("espesorCavidad", user_arguments)
    ratioAreaProyectada = runner.getDoubleArgumentValue("ratioAreaProyectada", user_arguments)
    efectividadHuecosRespectoAlViento = runner.getDoubleArgumentValue("efectividadHuecosRespectoAlViento", user_arguments)
    coeficientesDescargaHuecosFlotabilidad = runner.getDoubleArgumentValue("coeficientesDescargaHuecosFlotabilidad", user_arguments)


    
    
    
    # listaSuperficies
    listaSuperficies = workspace.getObjectsByType("BuildingSurface_Detailed".to_IddObjectType)
    
    listaSuperficies = listaSuperficies.select { |x| x.getString(4).get == "Outdoors" }

    # reporting initial condition of model
    runner.registerInitialCondition("El modelo contiene  #{listaSuperficies.size} superficies exteriores.")


    sideConditionModel = "
    SurfaceProperty:OtherSideConditionsModel,
         Cavidad Fachada Ventilada,          ! OtherSideConditionsModel Name
         GapConvectionRadiation;     ! Type of Modeling used to determine Boundary Conditions
        "
    idfObject = OpenStudio::IdfObject::load(sideConditionModel)
    object = idfObject.get
    wsObject = workspace.addObject(object)

    # Aplicamos a las superficies las nuevas condiciones de contorno
    
    nombresDeLasSuperficies = []
    listaSuperficies.each do |superficie|
      superficie.setString(4,"OtherSideConditionsModel")
      superficie.setString(5,"Cavidad Fachada Ventilada")
      
      nombresDeLasSuperficies << superficie.getString(0).get
    end
    
    
    naturallyVentedCavity = "
    SurfaceProperty:ExteriorNaturalVentedCavity,
        Vented Cavity Exterior Surface,     ! Name
        Cavidad Fachada Ventilada,          ! OtherSideConditionsModel Object Name
        #{fraccionAreaAperturas},                       ! Area Fraction of Openings
        #{emisividadPlacaExterior},                        ! Thermal Emissivity of Exterior Baffle Material
        #{absortividadSolarPlacaExterior},                       ! Solar Absorbtivity of Exterior Baffle
        #{alturaMediaParaVentilacionPorFlotabilidad},                       ! Height scale for bouyancy-driven ventilation
        #{espesorCavidad},                       ! Effective Thickness of Cavity Behind Exterior Baffle
        #{ratioAreaProyectada},                       ! Ratio of Actual surface area to projected surface area
        Smooth ,                    ! Roughness of collector
        #{efectividadHuecosRespectoAlViento} ,                       ! Cv, Effectiveness for perforations with respect to Wind
        #{coeficientesDescargaHuecosFlotabilidad} ,                       ! Cd, Discharge Coefficient for Openings with respect to bouyancy-driven flow
        #{nombresDeLasSuperficies.join(",")} ;   ! Surface Name
        "
    
    cavity = OpenStudio::IdfObject::load(naturallyVentedCavity).get
    wsCavity = workspace.addObject(cavity)



    runner.registerFinalCondition("Se han modificado  #{listaSuperficies.size} superficies.")
    
    return true
 
  end

end 

# register the measure to be used by the application
IntroduceUnaFachadaVentilada.new.registerWithApplication
